* #define BITR2L
* ========================================================================= *
* This is an original work of Peter Townsend and is hereby placed into the  *
* public domain.                                                            *
* E-mail: cephas@tpgi.com.au                                                *
* ========================================================================= *
* WARNING!  Some of these functions assume that bits 1-8 run left to right, *
*           not right to left as is normally the case. This allows for long *
*           strings of bits to be contiguous (ie, bit 9 is next to bit 8).  *
*           For instance, I store a logical value for every day of the      *
*           year in 46 bytes (1=.T., 0=.F., 366 days / 8 bits = 46 bytes).  *
*           If you wish to use the functions in a right to left context,    *
*           uncomment the #define at the start of the file.                 *
* ========================================================================= *
* ByteAnd  - AND two bytes together.                                        *
*                                                                           *
* Parameters: <ExpC1> - first byte.                                         *
*             <ExpC2> - second byte.                                        *
*                                                                           *
* Returns   : <ExpC>  - the result of the AND.                              *
* ------------------------------------------------------------------------- *
FUNCTION ByteAnd(cByte1, cByte2)
    LOCAL nCntr, nRetVal
    nRetVal := 0
    FOR nCntr := 1 TO 8
        nRetVal *= 2
#ifdef BITR2L
        IF BitOn(cByte1, 9 - nCntr) .AND. BitOn(cByte2, 9 - nCntr)
#else
        IF BitOn(cByte1, nCntr) .AND. BitOn(cByte2, nCntr)
#endif
            nRetVal += 1
        ENDIF
    NEXT
RETURN(CHR(nRetVal))
* ========================================================================= *
* ByteOr   - OR two bytes together.                                         *
*                                                                           *
* Parameters: <ExpC1> - first byte.                                         *
*             <ExpC2> - second byte.                                        *
*                                                                           *
* Returns   : <ExpC>  - the result of the OR.                               *
* ------------------------------------------------------------------------- *
FUNCTION ByteOr(cByte1, cByte2)
    LOCAL nCntr, nRetVal
    nRetVal := 0
    FOR nCntr := 1 TO 8
        nRetVal *= 2
#ifdef BITR2L
        IF BitOn(cByte1, 9 - nCntr) .OR. BitOn(cByte2, 9 - nCntr)
#else
        IF BitOn(cByte1, nCntr) .OR. BitOn(cByte2, nCntr)
#endif
            nRetVal += 1
        ENDIF
    NEXT
RETURN(CHR(nRetVal))
* ========================================================================= *
* ByteXOr   - XOR two bytes together.                                       *
*                                                                           *
* Parameters: <ExpC1> - first byte.                                         *
*             <ExpC2> - second byte.                                        *
*                                                                           *
* Returns   : <ExpC>  - the result of the XOR.                              *
* ------------------------------------------------------------------------- *
FUNCTION ByteXOr(cByte1, cByte2)
    LOCAL nCntr, nRetVal, lOneOn, lTwoOn
    nRetVal := 0
    FOR nCntr := 1 TO 8
        nRetVal *= 2
#ifdef BITR2L
        lOneOn  := BitOn(cByte1, 9 - nCntr)
        lTwoOn  := BitOn(cByte2, 9 - nCntr)
#else
        lOneOn  := BitOn(cByte1, nCntr)
        lTwoOn  := BitOn(cByte2, nCntr)
#endif
        IF (lOneOn .AND. (! lTwoOn)) .OR. ((! lOneOn) .AND. lTwoOn)
            nRetVal += 1
        ENDIF
    NEXT
RETURN(CHR(nRetVal))
* ========================================================================= *
* BitOn   - Is the nth bit of a byte a 1?                                   *
*                                                                           *
* Parameters: <ExpC1> - byte to check                                       *
*             <ExpN1> - number of bit to check                              *
*                                                                           *
* Returns   : <ExpL>  - Is the nth bit on?                                  *
* ------------------------------------------------------------------------- *
FUNCTION BitOn(cByte, nPosn)
    LOCAL nVal
#ifdef BITR2L
    nPosn := 9 - nPosn
#endif
    nVal  := INT(ASC(cByte) / (2^(8-nPosn)))
RETURN(MOD(nVal, 2) == 1)
* ========================================================================= *
* BitOff  - Is the nth bit of a byte a 0?                                   *
*                                                                           *
* Parameters: <ExpC1> - byte to check                                       *
*             <ExpN1> - number of bit to check                              *
*                                                                           *
* Returns   : <ExpL>  - Is the nth bit off?                                 *
* ------------------------------------------------------------------------- *
FUNCTION BitOff(cByte, nPosn)
RETURN(! BitOn(cByte, nPosn))
* ========================================================================= *
* ByteLftSh   - Left shift the bits of a byte by n bits                     *
*                                                                           *
* Parameters: <ExpC1> - byte to left shift                                  *
*             <ExpN1> - number of bits by which to move byte left           *
*                                                                           *
* Returns   : <ExpC>  - The left-shifted byte                               *
* ------------------------------------------------------------------------- *
FUNCTION ByteLftSh(cByte, nPosns)
    LOCAL nVal
    nVal := MOD((ASC(cByte) * (2^nPosns)), 256)
RETURN(CHR(nVal))
* ========================================================================= *
* ByteRytSh   - Right shift the bits of a byte by n bits                    *
*                                                                           *
* Parameters: <ExpC1> - byte to right shift                                 *
*             <ExpN1> - number of bits by which to move byte right          *
*                                                                           *
* Returns   : <ExpC>  - The right-shifted byte                              *
* ------------------------------------------------------------------------- *
FUNCTION ByteRytSh(cByte, nPosns)
    LOCAL nVal
    nVal := INT((ASC(cByte) / (2^nPosns)))
RETURN(CHR(nVal))
* ========================================================================= *
* BitTurnOn - Turn on the nth bit of a byte                                 *
*                                                                           *
* Parameters: <ExpC1> - byte to manipulate                                  *
*             <ExpN1> - the number of the bit to turn on                    *
*                                                                           *
* Returns   : <ExpC>  - The manipulated byte                                *
* ------------------------------------------------------------------------- *
FUNCTION BitTurnOn(cByte, nPosn)
#ifdef BITR2L
    nPosn := 9 - nPosn
#endif
RETURN(ByteOr(cByte, CHR(2^(8-nPosn))))
* ========================================================================= *
* BitTurnOff - Turn off the nth bit of a byte                               *
*                                                                           *
* Parameters: <ExpC1> - byte to manipulate                                  *
*             <ExpN1> - the number of the bit to turn off                   *
*                                                                           *
* Returns   : <ExpC>  - The manipulated byte                                *
* ------------------------------------------------------------------------- *
FUNCTION BitTurnOff(cByte, nPosn)
#ifdef BITR2L
    nPosn := 9 - nPosn
#endif
RETURN(ByteAnd(cByte, CHR(255 - (2^(8-nPosn)))))
* ========================================================================= *
* BitFlip - Flip the nth bit of a byte                                      *
*                                                                           *
* Parameters: <ExpC1> - byte to manipulate                                  *
*             <ExpN1> - the number of the bit to flip                       *
*                                                                           *
* Returns   : <ExpC>  - The manipulated byte                                *
* ------------------------------------------------------------------------- *
FUNCTION BitFlip(cByte, nPosn)
RETURN(IF(BitOn(cByte, nPosn), BitTurnOff(cByte, nPosn), BitTurnOn(cByte, nPosn)))
* ========================================================================= *
* ByteFlip - Flip all of the bits in a byte                                 *
*                                                                           *
* Parameters: <ExpC1> - byte to manipulate                                  *
*                                                                           *
* Returns   : <ExpC>  - The manipulated byte                                *
* ------------------------------------------------------------------------- *
FUNCTION ByteFlip(cByte)
    LOCAL nCntr, cRetVal
    cRetVal := cByte
    FOR nCntr := 1 TO 8
        cRetVal := BitFlip(cRetVal, nCntr)
    NEXT
RETURN(cRetVal)
* ========================================================================= *
* ByteShow - Convert each bit of a byte into a byte for display             *
*                                                                           *
* Parameters: <ExpC1> - byte to display                                     *
*                                                                           *
* Returns   : <ExpS>  - An 8-byte string                                    *
* ------------------------------------------------------------------------- *
FUNCTION ByteShow(cByte)
    LOCAL nCntr, cByteStr := ""
    FOR nCntr := 1 TO 8
#ifdef BITR2L
        cByteStr += IF(BitOn(cByte, 9 - nCntr), "1", "0")
#else
        cByteStr += IF(BitOn(cByte, nCntr), "1", "0")
#endif
    NEXT
RETURN(cByteStr)
